"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.S3ObjectLambdaEventSchema = exports.S3SqsEventNotificationSchema = exports.S3EventNotificationEventBridgeSchema = exports.S3Schema = void 0;
const zod_1 = require("zod");
const eventbridge_js_1 = require("./eventbridge.js");
const sqs_js_1 = require("./sqs.js");
const S3Identity = zod_1.z.object({
    principalId: zod_1.z.string(),
});
const S3RequestParameters = zod_1.z.object({
    sourceIPAddress: zod_1.z.string().ip(),
});
const S3ResponseElements = zod_1.z.object({
    'x-amz-request-id': zod_1.z.string(),
    'x-amz-id-2': zod_1.z.string(),
});
const S3Message = zod_1.z.object({
    s3SchemaVersion: zod_1.z.string(),
    configurationId: zod_1.z.string(),
    object: zod_1.z.object({
        key: zod_1.z.string(),
        size: zod_1.z.number().optional(),
        urlDecodedKey: zod_1.z.string().optional(),
        eTag: zod_1.z.string().optional(),
        sequencer: zod_1.z.string(),
        versionId: zod_1.z.optional(zod_1.z.string()),
    }),
    bucket: zod_1.z.object({
        name: zod_1.z.string(),
        ownerIdentity: S3Identity,
        arn: zod_1.z.string(),
    }),
});
const S3EventRecordGlacierEventData = zod_1.z.object({
    restoreEventData: zod_1.z.object({
        lifecycleRestorationExpiryTime: zod_1.z.string(),
        lifecycleRestoreStorageClass: zod_1.z.string(),
    }),
});
const S3RecordSchema = zod_1.z
    .object({
    eventVersion: zod_1.z.string(),
    eventSource: zod_1.z.literal('aws:s3'),
    awsRegion: zod_1.z.string(),
    eventTime: zod_1.z.string().datetime(),
    eventName: zod_1.z.string(),
    userIdentity: S3Identity,
    requestParameters: S3RequestParameters,
    responseElements: S3ResponseElements,
    s3: S3Message,
    glacierEventData: zod_1.z.optional(S3EventRecordGlacierEventData),
})
    .refine((value) => {
    return ((!value.eventName.includes('ObjectRemoved') &&
        value.s3.object.size === undefined) ||
        value.s3.object.eTag === undefined,
        {
            message: 'S3 event notification with ObjectRemoved event name must have size or eTag defined',
        });
});
const S3EventNotificationEventBridgeDetailSchema = zod_1.z.object({
    version: zod_1.z.string(),
    bucket: zod_1.z.object({
        name: zod_1.z.string(),
    }),
    object: zod_1.z.object({
        key: zod_1.z.string(),
        size: zod_1.z.number().nonnegative().optional(),
        etag: zod_1.z.string(),
        'version-id': zod_1.z.string().optional(),
        sequencer: zod_1.z.string().optional(),
    }),
    'request-id': zod_1.z.string(),
    requester: zod_1.z.string(),
    'source-ip-address': zod_1.z.string().ip().optional(),
    reason: zod_1.z.string().optional(),
    'deletion-type': zod_1.z.string().optional(),
    'restore-expiry-time': zod_1.z.string().optional(),
    'source-storage-class': zod_1.z.string().optional(),
    'destination-storage-class': zod_1.z.string().optional(),
    'destination-access-tier': zod_1.z.string().optional(),
});
const S3EventNotificationEventBridgeSchema = eventbridge_js_1.EventBridgeSchema.extend({
    detail: S3EventNotificationEventBridgeDetailSchema,
});
exports.S3EventNotificationEventBridgeSchema = S3EventNotificationEventBridgeSchema;
const S3Schema = zod_1.z.object({
    Records: zod_1.z.array(S3RecordSchema),
});
exports.S3Schema = S3Schema;
const S3SqsEventNotificationRecordSchema = sqs_js_1.SqsRecordSchema.extend({
    body: zod_1.z.string(),
});
const S3SqsEventNotificationSchema = zod_1.z.object({
    Records: zod_1.z.array(S3SqsEventNotificationRecordSchema),
});
exports.S3SqsEventNotificationSchema = S3SqsEventNotificationSchema;
const S3ObjectContext = zod_1.z.object({
    inputS3Url: zod_1.z.string().url(),
    outputRoute: zod_1.z.string(),
    outputToken: zod_1.z.string(),
});
const S3ObjectConfiguration = zod_1.z.object({
    accessPointArn: zod_1.z.string(),
    supportingAccessPointArn: zod_1.z.string(),
    payload: zod_1.z.union([zod_1.z.string(), zod_1.z.object({})]),
});
const S3ObjectUserRequest = zod_1.z.object({
    url: zod_1.z.string(),
    headers: zod_1.z.record(zod_1.z.string(), zod_1.z.string()),
});
const S3ObjectSessionContext = zod_1.z.object({
    sessionIssuer: zod_1.z.object({
        type: zod_1.z.string(),
        userName: zod_1.z.string().optional(),
        principalId: zod_1.z.string(),
        arn: zod_1.z.string(),
        accountId: zod_1.z.string(),
    }),
    attributes: zod_1.z.object({
        creationDate: zod_1.z.string(),
        mfaAuthenticated: zod_1.z
            .union([zod_1.z.boolean(), zod_1.z.literal('true'), zod_1.z.literal('false')])
            .transform((value) => value === true || value === 'true'),
    }),
});
const S3ObjectUserIdentity = zod_1.z.object({
    type: zod_1.z.string(),
    accountId: zod_1.z.string(),
    accessKeyId: zod_1.z.string(),
    userName: zod_1.z.string().optional(),
    principalId: zod_1.z.string(),
    arn: zod_1.z.string(),
    sessionContext: S3ObjectSessionContext.optional(),
});
const S3ObjectLambdaEventSchema = zod_1.z.object({
    xAmzRequestId: zod_1.z.string(),
    getObjectContext: S3ObjectContext,
    configuration: S3ObjectConfiguration,
    userRequest: S3ObjectUserRequest,
    userIdentity: S3ObjectUserIdentity,
    protocolVersion: zod_1.z.string(),
});
exports.S3ObjectLambdaEventSchema = S3ObjectLambdaEventSchema;
